"""
Channel level settings (global for all channels).
"""


class ChannelSettingsMixin:
    """
    Mixin for channel settings that apply to all channels.
    Provides common settings for all channels. Each channels is free to override settings at their
    Exporter or Transmitter or Client level, as needed

    Important: If you add a setting here, please add a test to cover this default, as well as
    any overrides you add on a per channel basis. See this test for an example:
    tests/test_integrated_channels/test_sap_success_factors/test_transmitters/test_learner_data.py
    """

    # a channel should override this to False if they don't want grade changes to
    # cause retransmission of completion records
    INCLUDE_GRADE_FOR_COMPLETION_AUDIT_CHECK = True



# Generated by Django 2.2.17 on 2020-12-23 18:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0117_auto_20201215_0258'),
    ]

    operations = [
        migrations.AlterField(
            model_name='historicalpendingenterprisecustomeruser',
            name='user_email',
            field=models.EmailField(max_length=254),
        ),
        migrations.AlterField(
            model_name='pendingenterprisecustomeruser',
            name='user_email',
            field=models.EmailField(max_length=254),
        ),
        migrations.AlterUniqueTogether(
            name='pendingenterprisecustomeradminuser',
            unique_together=set(),
        ),
        migrations.AddIndex(
            model_name='pendingenterprisecustomeradminuser',
            index=models.Index(fields=['user_email', 'enterprise_customer'], name='enterprise__user_em_fead22_idx'),
        ),
        migrations.AddIndex(
            model_name='pendingenterprisecustomeradminuser',
            index=models.Index(fields=['user_email'], name='enterprise__user_em_6e1f5b_idx'),
        ),
        migrations.AddIndex(
            model_name='pendingenterprisecustomeruser',
            index=models.Index(fields=['user_email', 'enterprise_customer'], name='enterprise__user_em_f98d36_idx'),
        ),
        migrations.AddIndex(
            model_name='pendingenterprisecustomeruser',
            index=models.Index(fields=['user_email'], name='enterprise__user_em_488930_idx'),
        ),
        migrations.AddConstraint(
            model_name='pendingenterprisecustomeradminuser',
            constraint=models.UniqueConstraint(fields=('user_email', 'enterprise_customer'), name='unique pending admin user and EnterpriseCustomer'),
        ),
        migrations.AddConstraint(
            model_name='pendingenterprisecustomeruser',
            constraint=models.UniqueConstraint(fields=('user_email', 'enterprise_customer'), name='unique user and EnterpriseCustomer'),
        ),
    ]



"""
Django management command to reencrypt passwords in enterprise custom reporting configs.
"""
import logging

from django.core.management import BaseCommand

from enterprise.models import EnterpriseCustomerReportingConfiguration

LOGGER = logging.getLogger(__name__)


class Command(BaseCommand):
    """
    Django management command to reencrypt passwords in enterprise custom reporting configs
    It's useful when following encryption keys are rotated
    - FERNET_KEYS
    - LMS_FERNET_KEY

    Example usage:
    ./manage.py lms reencrypt_enterprise_customer_reporting_config_passwords

    """
    def handle(self, *args, **options):
        try:
            for config in EnterpriseCustomerReportingConfiguration.objects.all():
                config.save()  # resaving reencrypts all the encrypted columns
            LOGGER.info('Enterprise customer reporting configuration passwords reencrypted succesfully!')
        except Exception as e:  # pylint: disable=broad-except
            LOGGER.exception(f'Failed to reencrypt customer reporting configuration passwords. Error: {e}')



# Generated by Django 1.11.23 on 2019-09-18 11:58


import jsonfield.fields

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cornerstone', '0003_auto_20190621_1000'),
    ]

    operations = [
        migrations.AddField(
            model_name='cornerstoneglobalconfiguration',
            name='languages',
            field=jsonfield.fields.JSONField(default={}, help_text='List of IETF language tags supported by cornerstone'),
        ),
    ]



"""
Generic learner data transmitter for integrated channels.
"""

import logging
from http import HTTPStatus

from django.apps import apps

from enterprise.utils import localized_utcnow
from integrated_channels.exceptions import ClientError
from integrated_channels.integrated_channel.channel_settings import ChannelSettingsMixin
from integrated_channels.integrated_channel.client import IntegratedChannelApiClient
from integrated_channels.integrated_channel.exporters.learner_data import LearnerExporterUtility
from integrated_channels.integrated_channel.transmitters import Transmitter
from integrated_channels.utils import encode_data_for_logging, generate_formatted_log, is_already_transmitted

LOGGER = logging.getLogger(__name__)


class LearnerTransmitter(Transmitter, ChannelSettingsMixin):
    """
    A generic learner data transmitter.

    It may be subclassed by specific integrated channel learner data transmitters for
    each integrated channel's particular learner data transmission requirements and expectations.
    """

    def __init__(self, enterprise_configuration, client=IntegratedChannelApiClient):
        """
        By default, use the abstract integrated channel API client which raises an error when used if not subclassed.
        """
        super().__init__(
            enterprise_configuration=enterprise_configuration,
            client=client
        )

    def _generate_common_params(self, **kwargs):
        """ Pulls labeled common params out of kwargs """
        app_label = kwargs.get('app_label', 'integrated_channel')
        enterprise_customer_uuid = self.enterprise_configuration.enterprise_customer.uuid or None
        lms_user_id = kwargs.get('learner_to_transmit', None)
        return app_label, enterprise_customer_uuid, lms_user_id

    def single_learner_assessment_grade_transmit(self, exporter, **kwargs):
        """
        Send an assessment level grade information to the integrated channel using the client.

        Args:
            exporter: The ``LearnerExporter`` instance used to send to the integrated channel.
            kwargs: Contains integrated channel-specific information for customized transmission variables.
                - app_label: The app label of the integrated channel for whom to store learner data records for.
                - model_name: The name of the specific learner data record model to use.
                - remote_user_id: The remote ID field name on the audit model that will map to the learner.
        """
        app_label, enterprise_customer_uuid, lms_user_id = self._generate_common_params(**kwargs)
        TransmissionAudit = apps.get_model(
            app_label=app_label,
            model_name=kwargs.get('model_name', 'GenericLearnerDataTransmissionAudit'),
        )
        kwargs.update(
            TransmissionAudit=TransmissionAudit,
        )
        kwargs.update(channel_name=app_label)

        if self.enterprise_configuration.disable_learner_data_transmissions:
            LOGGER.info(generate_formatted_log(
                self.enterprise_configuration.channel_code(),
                enterprise_customer_uuid,
                lms_user_id,
                None,
                "Single learner data assessment level transmission skipped as customer's configuration has marked "
                "learner data reporting as disabled."
            ))
            return

        # Even though we're transmitting a learner, they can have records per assessment (multiple per course).
        for learner_data in exporter.single_assessment_level_export(**kwargs):
            LOGGER.info(generate_formatted_log(
                self.enterprise_configuration.channel_code(),
                enterprise_customer_uuid,
                lms_user_id,
                learner_data.course_id,
                'create_assessment_reporting started for '
                f'integrated_channel_enterprise_enrollment_id={learner_data.enterprise_course_enrollment_id}'
            ))

            serialized_payload = learner_data.serialize(enterprise_configuration=self.enterprise_configuration)

            if self.enterprise_configuration.dry_run_mode_enabled:
                remote_id = getattr(learner_data, kwargs.get('remote_user_id'))
                encoded_serialized_payload = encode_data_for_logging(serialized_payload)
                LOGGER.info(generate_formatted_log(
                    self.enterprise_configuration.channel_code(),
                    enterprise_customer_uuid,
                    lms_user_id,
                    learner_data.course_id,
                    'dry-run mode '
                    'skipping create_assessment_reporting for enrollment '
                    f'integrated_channel_enterprise_enrollment_id={learner_data.enterprise_course_enrollment_id}, '
                    f'integrated_channel_remote_user_id={remote_id}, '
                    f'integrated_channel_serialized_payload_base64={encoded_serialized_payload}'
                ))
                continue

            try:
                code, body = self.client.create_assessment_reporting(
                    getattr(learner_data, kwargs.get('remote_user_id')),
                    serialized_payload
                )
            except ClientError as client_error:
                code = client_error.status_code
                body = client_error.message
                self.process_transmission_error(
                    learner_data,
                    client_error,
                    app_label,
                    enterprise_customer_uuid,
                    lms_user_id,
                    learner_data.course_id
                )

            except Exception:
                # Log additional data to help debug failures but just have Exception bubble
                self._log_exception_supplemental_data(
                    learner_data,
                    'create_assessment_reporting',
                    app_label,
                    enterprise_customer_uuid,
                    lms_user_id,
                    learner_data.course_id
                )
                raise

            learner_data.status = str(code)
            learner_data.error_message = body if code >= 400 else ''
            learner_data.save()

    def assessment_level_transmit(self, exporter, **kwargs):
        """
        Send all assessment level grade information under an enterprise enrollment to the integrated channel using the
        client.

        Args:
            exporter: The learner assessment data exporter used to send to the integrated channel.
            kwargs: Contains integrated channel-specific information for customized transmission variables.
                - app_label: The app label of the integrated channel for whom to store learner data records for.
                - model_name: The name of the specific learner data record model to use.
                - remote_user_id: The remote ID field name of the learner on the audit model.
        """
        app_label, enterprise_customer_uuid, _ = self._generate_common_params(**kwargs)
        TransmissionAudit = apps.get_model(
            app_label=app_label,
            model_name=kwargs.get('model_name', 'GenericLearnerDataTransmissionAudit'),
        )
        kwargs.update(
            TransmissionAudit=TransmissionAudit,
        )

        if self.enterprise_configuration.disable_learner_data_transmissions:
            LOGGER.info(generate_formatted_log(
                self.enterprise_configuration.channel_code(),
                enterprise_customer_uuid,
                None,
                None,
                "Bulk learner data assessment level transmission skipped as customer's configuration has marked "
                "learner data reporting as disabled."
            ))
            return

        # Retrieve learner data for each existing enterprise enrollment under the enterprise customer
        # and transmit the data according to the current enterprise configuration.
        for learner_data in exporter.bulk_assessment_level_export():
            serialized_payload = learner_data.serialize(enterprise_configuration=self.enterprise_configuration)
            enterprise_enrollment_id = learner_data.enterprise_course_enrollment_id
            lms_user_id = LearnerExporterUtility.lms_user_id_for_ent_course_enrollment_id(
                enterprise_enrollment_id
            )

            # Check the last transmission for the current enrollment and see if the old grades match the new ones
            # note: the property self.include_grade_for_completion_audit_check comes from ChannelSettingsMixin
            if is_already_transmitted(
                TransmissionAudit,
                enterprise_enrollment_id,
                self.enterprise_configuration.id,
                learner_data.grade,
                learner_data.subsection_id,
                detect_grade_updated=self.INCLUDE_GRADE_FOR_COMPLETION_AUDIT_CHECK,
            ):
                # We've already sent a completion status for this enrollment
                LOGGER.info(generate_formatted_log(
                    self.enterprise_configuration.channel_code(),
                    enterprise_customer_uuid,
                    lms_user_id,
                    learner_data.course_id,
                    'Skipping previously sent '
                    f'integrated_channel_enterprise_enrollment_id={enterprise_enrollment_id}'
                ))
                continue

            if self.enterprise_configuration.dry_run_mode_enabled:
                remote_id = getattr(learner_data, kwargs.get('remote_user_id'))
                encoded_serialized_payload = encode_data_for_logging(serialized_payload)
                LOGGER.info(generate_formatted_log(
                    self.enterprise_configuration.channel_code(),
                    enterprise_customer_uuid,
                    lms_user_id,
                    learner_data.course_id,
                    'dry-run mode '
                    'skipping create_assessment_reporting for enrollment '
                    f'integrated_channel_enterprise_enrollment_id={enterprise_enrollment_id}, '
                    f'integrated_channel_remote_user_id={remote_id}, '
                    f'integrated_channel_serialized_payload_base64={encoded_serialized_payload}'
                ))
                continue

            try:
                code, body = self.client.create_assessment_reporting(
                    getattr(learner_data, kwargs.get('remote_user_id')),
                    serialized_payload
                )
                LOGGER.info(generate_formatted_log(
                    self.enterprise_configuration.channel_code(),
                    enterprise_customer_uuid,
                    lms_user_id,
                    learner_data.course_id,
                    'create_assessment_reporting successfully completed for '
                    f'integrated_channel_enterprise_enrollment_id={enterprise_enrollment_id}'
                ))
            except ClientError as client_error:
                code = client_error.status_code
                body = client_error.message
                self.process_transmission_error(
                    learner_data,
                    client_error,
                    app_label,
                    enterprise_customer_uuid,
                    lms_user_id,
                    learner_data.course_id
                )

            except Exception:
                # Log additional data to help debug failures but just have Exception bubble
                self._log_exception_supplemental_data(
                    learner_data,
                    'create_assessment_reporting',
                    app_label,
                    enterprise_customer_uuid,
                    lms_user_id,
                    learner_data.course_id
                )
                raise

            learner_data.status = str(code)
            learner_data.error_message = body if code >= 400 else ''
            learner_data.save()

    def transmit(self, payload, **kwargs):  # pylint: disable=arguments-differ
        """
        Send a completion status call to the integrated channel using the client.

        Args:
            payload: The learner data exporter.
            kwargs: Contains integrated channel-specific information for customized transmission variables.
                - app_label: The app label of the integrated channel for whom to store learner data records for.
                - model_name: The name of the specific learner data record model to use.
                - remote_user_id: The remote ID field name of the learner on the audit model.
        """
        app_label, enterprise_customer_uuid, _ = self._generate_common_params(**kwargs)
        TransmissionAudit = apps.get_model(
            app_label=app_label,
            model_name=kwargs.get('model_name', 'GenericLearnerDataTransmissionAudit'),
        )
        kwargs.update(
            TransmissionAudit=TransmissionAudit,
        )

        if self.enterprise_configuration.disable_learner_data_transmissions:
            LOGGER.info(generate_formatted_log(
                self.enterprise_configuration.channel_code(),
                enterprise_customer_uuid,
                None,
                None,
                "Completion level learner data transmission skipped as customer's configuration has marked learner data"
                " reporting as disabled."
            ))
            return

        # Since we have started sending courses to integrated channels instead of course runs,
        # we need to attempt to send transmissions with course keys and course run ids in order to
        # ensure that we account for whether courses or course runs exist in the integrated channel.
        # The exporters have been changed to return multiple transmission records to attempt,
        # one by course key and one by course run id.
        # If the transmission with the course key succeeds, the next one will get skipped.
        # If it fails, the one with the course run id will be attempted and (presumably) succeed.

        for learner_data in payload.export(**kwargs):
            serialized_payload = learner_data.serialize(enterprise_configuration=self.enterprise_configuration)

            enterprise_enrollment_id = learner_data.enterprise_course_enrollment_id
            lms_user_id = LearnerExporterUtility.lms_user_id_for_ent_course_enrollment_id(
                enterprise_enrollment_id
            )

            if (not learner_data.course_completed and
                    not getattr(self.enterprise_configuration, 'enable_incomplete_progress_transmission', False)):
                # The user has not completed the course and enable_incomplete_progress_transmission is not set,
                # so we shouldn't send a completion status call
                remote_id = getattr(learner_data, kwargs.get('remote_user_id'))
                encoded_serialized_payload = encode_data_for_logging(serialized_payload)
                continue

            grade = getattr(learner_data, 'grade', None)
            if is_already_transmitted(
                TransmissionAudit,
                enterprise_enrollment_id,
                self.enterprise_configuration.id,
                grade,
                detect_grade_updated=self.INCLUDE_GRADE_FOR_COMPLETION_AUDIT_CHECK,
            ):
                # We've already sent a completion status for this enrollment
                continue

            if self.enterprise_configuration.dry_run_mode_enabled:
                remote_id = getattr(learner_data, kwargs.get('remote_user_id'))
                encoded_serialized_payload = encode_data_for_logging(serialized_payload)
                LOGGER.info(generate_formatted_log(
                    self.enterprise_configuration.channel_code(),
                    enterprise_customer_uuid,
                    lms_user_id,
                    learner_data.course_id,
                    'dry-run mode '
                    f'integrated_channel_enterprise_enrollment_id={enterprise_enrollment_id}, '
                    f'integrated_channel_remote_user_id={remote_id}, '
                    f'integrated_channel_serialized_payload_base64={encoded_serialized_payload}'
                ))
                continue

            try:
                code, body = self.client.create_course_completion(
                    getattr(learner_data, kwargs.get('remote_user_id')),
                    serialized_payload
                )
                if code >= HTTPStatus.BAD_REQUEST.value:
                    raise ClientError(f'Client create_course_completion failed: {body}', code)

                LOGGER.info(generate_formatted_log(
                    self.enterprise_configuration.channel_code(),
                    enterprise_customer_uuid,
                    lms_user_id,
                    learner_data.course_id,
                    'Successfully sent completion status call for enterprise enrollment '
                    f'integrated_channel_enterprise_enrollment_id={enterprise_enrollment_id}'
                ))
            except ClientError as client_error:
                code = client_error.status_code
                body = client_error.message
                self.process_transmission_error(
                    learner_data,
                    client_error,
                    app_label,
                    enterprise_customer_uuid,
                    lms_user_id,
                    learner_data.course_id
                )

            except Exception:
                # Log additional data to help debug failures but have Exception bubble
                self._log_exception_supplemental_data(
                    learner_data,
                    'create_assessment_reporting',
                    app_label,
                    enterprise_customer_uuid,
                    lms_user_id,
                    learner_data.course_id
                )
                raise

            action_happened_at = localized_utcnow()
            was_successful = code < 300
            learner_data.status = str(code)
            learner_data.error_message = body if not was_successful else ''
            learner_data.save()
            self.enterprise_configuration.update_learner_synced_at(action_happened_at, was_successful)

    def deduplicate_assignment_records_transmit(self, exporter, **kwargs):
        """
        Remove duplicated assessments sent to the integrated channel using the client.

        Args:
            exporter: The learner completion data payload to send to the integrated channel.
            kwargs: Contains integrated channel-specific information for customized transmission variables.
                - app_label: The app label of the integrated channel for whom to store learner data records for.
                - model_name: The name of the specific learner data record model to use.
                - remote_user_id: The remote ID field name of the learner on the audit model.
        """
        app_label, enterprise_customer_uuid, _ = self._generate_common_params(**kwargs)
        courses = exporter.export_unique_courses()

        if self.enterprise_configuration.dry_run_mode_enabled:
            LOGGER.info(generate_formatted_log(
                self.enterprise_configuration.channel_code(),
                enterprise_customer_uuid,
                None,
                None,
                'dry-run mode '
                'skipping deduplicate_assignment_records_transmit'
            ))
            return

        code, body = self.client.cleanup_duplicate_assignment_records(courses)

        if code >= 400:
            LOGGER.exception(
                generate_formatted_log(
                    self.enterprise_configuration.channel_code(),
                    enterprise_customer_uuid,
                    None,
                    None,
                    f'{app_label} Deduping assignments transmission experienced a failure, '
                    f'received the error message: {body}'
                )
            )
        else:
            LOGGER.info(
                generate_formatted_log(
                    self.enterprise_configuration.channel_code(),
                    enterprise_customer_uuid,
                    None,
                    None,
                    f'{app_label} Deduping assignments transmission finished successfully, '
                    f'received message: {body}'
                )
            )

    def _log_exception_supplemental_data(self, learner_data, operation_name,
                                         integrated_channel_name, enterprise_customer_uuid, learner_id, course_id):
        """ Logs extra payload and parameter data to help debug which learner data caused an exception. """
        LOGGER.exception(generate_formatted_log(
            self.enterprise_configuration.channel_code(), enterprise_customer_uuid, learner_id, course_id,
            '{operation_name} {integrated_channel_name} failed with Exception for '
            'enterprise enrollment {enrollment_id} with payload {payload}'.format(
                operation_name=operation_name,
                integrated_channel_name=integrated_channel_name,
                enrollment_id=learner_data.enterprise_course_enrollment_id,
                payload=learner_data
            )), exc_info=True)

    def handle_transmission_error(self, learner_data, client_exception):
        """
        Subclasses who wish to do additional processing of transmission error
        before logging, can do so in overrides of this method
        """

    def process_transmission_error(
        self,
        learner_data,
        client_exception,
        integrated_channel_name,
        enterprise_customer_uuid,
        learner_id,
        course_id,
    ):
        """
        applies any needed processing of transmission error before logging it
        subclasses should override handle_transmission_error if they want to do additional
        processing of the error before it's logged
        """
        self.handle_transmission_error(
            learner_data,
            client_exception,
        )
        serialized_payload = learner_data.serialize(enterprise_configuration=self.enterprise_configuration)
        encoded_serialized_payload = encode_data_for_logging(serialized_payload)
        LOGGER.exception(
            generate_formatted_log(
                self.enterprise_configuration.channel_code(), enterprise_customer_uuid, learner_id, course_id,
                f'Failed to send completion status call for {integrated_channel_name} '
                f'integrated_channel_enterprise_enrollment_id={learner_data.enterprise_course_enrollment_id}, '
                f'integrated_channel_serialized_payload_base64={encoded_serialized_payload}, '
                f'Error message: {client_exception.message} '
                f'Error status code: {client_exception.status_code}'
            )
        )



"""
Generic content metadata transmitter for integrated channels.
"""

import functools
import json
import logging
from itertools import islice

import requests

from django.apps import apps
from django.conf import settings

from enterprise.utils import localized_utcnow, truncate_string
from integrated_channels.exceptions import ClientError
from integrated_channels.integrated_channel.client import IntegratedChannelApiClient
from integrated_channels.integrated_channel.transmitters import Transmitter
from integrated_channels.utils import chunks, encode_binary_data_for_logging, generate_formatted_log

LOGGER = logging.getLogger(__name__)


class ContentMetadataTransmitter(Transmitter):
    """
    Used to transmit content metadata to an integrated channel.
    """

    # a 'magic number' to designate an unknown error
    UNKNOWN_ERROR_HTTP_STATUS_CODE = 555

    def __init__(self, enterprise_configuration, client=IntegratedChannelApiClient):
        """
        By default, use the abstract integrated channel API client which raises an error when used if not subclassed.
        """
        super().__init__(
            enterprise_configuration=enterprise_configuration,
            client=client
        )
        self._transmit_create = functools.partial(
            self._transmit_action,
            client_method=self.client.create_content_metadata,
            action_name='create',
        )
        self._transmit_update = functools.partial(
            self._transmit_action,
            client_method=self.client.update_content_metadata,
            action_name='update',
        )
        self._transmit_delete = functools.partial(
            self._transmit_action,
            client_method=self.client.delete_content_metadata,
            action_name='delete',
        )

    def _log_info(self, msg, course_or_course_run_key=None):
        LOGGER.info(
            generate_formatted_log(
                channel_name=self.enterprise_configuration.channel_code(),
                enterprise_customer_uuid=self.enterprise_configuration.enterprise_customer.uuid,
                course_or_course_run_key=course_or_course_run_key,
                plugin_configuration_id=self.enterprise_configuration.id,
                message=msg
            )
        )

    def _log_error(self, msg, course_or_course_run_key=None):
        LOGGER.error(
            generate_formatted_log(
                channel_name=self.enterprise_configuration.channel_code(),
                enterprise_customer_uuid=self.enterprise_configuration.enterprise_customer.uuid,
                course_or_course_run_key=course_or_course_run_key,
                plugin_configuration_id=self.enterprise_configuration.id,
                message=msg
            )
        )

    def _log_info_for_each_item_map(self, item_map, msg):
        for content_id, transmission in item_map.items():
            self._log_info(
                f'integrated_channel_content_transmission_id={transmission.id}, '
                f'{msg}',
                course_or_course_run_key=content_id
            )

    def transmit(self, create_payload, update_payload, delete_payload):
        """
        Transmit content metadata items to the integrated channel. Save or update content metadata records according to
        the type of transmission.
        """
        delete_payload_results = self._transmit_delete(delete_payload)

        create_payload_results = self._transmit_create(create_payload)

        update_payload_results = self._transmit_update(update_payload)
        return create_payload_results, update_payload_results, delete_payload_results

    def _prepare_items_for_transmission(self, channel_metadata_items):
        """
        Perform any necessary modifications to content metadata item
        data structure before transmission. This can be overridden by
        subclasses to add any data structure wrappers expected by the
        integrated channel.
        """
        return channel_metadata_items

    def _prepare_items_for_delete(self, channel_metadata_items):
        """
        Perform any necessary modifications to content metadata item
        data structure before delete. This can be overridden by
        subclasses to add any data structure wrappers expected by the
        integrated channel.
        """
        return channel_metadata_items

    def _serialize_items(self, channel_metadata_items):
        """
        Serialize content metadata items for a create transmission to the integrated channel.
        """
        return json.dumps(
            self._prepare_items_for_transmission(channel_metadata_items),
            sort_keys=True
        ).encode('utf-8')

    def _filter_api_response(self, response, content_id):  # pylint: disable=unused-argument
        """
        Filter the response from the integrated channel API client.
        This can be overridden by subclasses to parse the response
        expected by the integrated channel.
        """
        return response

    def _transmit_action(self, content_metadata_item_map, client_method, action_name):  # pylint: disable=too-many-statements
        """
        Do the work of calling the appropriate client method, saving the results, and updating
        the appropriate timestamps
        """
        results = []
        chunk_items = chunks(content_metadata_item_map, self.enterprise_configuration.transmission_chunk_size)
        transmission_limit = settings.INTEGRATED_CHANNELS_API_CHUNK_TRANSMISSION_LIMIT.get(
            self.enterprise_configuration.channel_code()
        )

        # If we're deleting, fetch all orphaned, unresolved content transmissions
        is_delete_action = action_name == 'delete'
        if is_delete_action:
            OrphanedContentTransmissions = apps.get_model(
                'integrated_channel',
                'OrphanedContentTransmissions'
            )
            orphaned_items = OrphanedContentTransmissions.objects.filter(
                integrated_channel_code=self.enterprise_configuration.channel_code(),
                plugin_configuration_id=self.enterprise_configuration.id,
                resolved=False,
            )
            successfully_removed_content_keys = []

        for chunk in islice(chunk_items, transmission_limit):
            json_payloads = [item.channel_metadata for item in list(chunk.values())]
            serialized_chunk = self._serialize_items(json_payloads)
            if self.enterprise_configuration.dry_run_mode_enabled:
                enterprise_customer_uuid = self.enterprise_configuration.enterprise_customer.uuid
                channel_code = self.enterprise_configuration.channel_code()
                for key, item in chunk.items():
                    payload = item.channel_metadata
                    serialized_payload = self._serialize_items([payload])
                    encoded_serialized_payload = encode_binary_data_for_logging(serialized_payload)
                    LOGGER.info(generate_formatted_log(
                        channel_code,
                        enterprise_customer_uuid,
                        None,
                        key,
                        f'dry-run mode content metadata '
                        f'skipping "{action_name}" action for content metadata transmission '
                        f'integrated_channel_serialized_payload_base64={encoded_serialized_payload}'
                    ))
                continue

            response_status_code = None
            response_body = None
            try:
                response_status_code, response_body = client_method(serialized_chunk)
            except ClientError as exc:
                LOGGER.exception(exc)
                response_status_code = exc.status_code
                response_body = str(exc)
                self._log_error(
                    f"Failed to {action_name} [{len(chunk)}] content metadata items for integrated channel "
                    f"[{self.enterprise_configuration.enterprise_customer.name}] "
                    f"[{self.enterprise_configuration.channel_code()}]. "
                    f"Task failed with message [{response_body}] and status code [{response_status_code}]"
                )
            except requests.exceptions.RequestException as exc:
                LOGGER.exception(exc)
                if exc.response:
                    response_status_code = exc.response.status_code
                    response_body = exc.response.text
                else:
                    response_status_code = self.UNKNOWN_ERROR_HTTP_STATUS_CODE
                    response_body = str(exc)
                self._log_error(
                    f"Failed to {action_name} [{len(chunk)}] content metadata items for integrated channel "
                    f"[{self.enterprise_configuration.enterprise_customer.name}] "
                    f"[{self.enterprise_configuration.channel_code()}]. "
                    f"Task failed with message [{str(exc)}] and status code [{response_status_code}]"
                )
            except Exception as exc:  # pylint: disable=broad-except
                LOGGER.exception(exc)
                response_status_code = self.UNKNOWN_ERROR_HTTP_STATUS_CODE
                response_body = str(exc)
                self._log_error(
                    f"Failed to {action_name} [{len(chunk)}] content metadata items for integrated channel "
                    f"[{self.enterprise_configuration.enterprise_customer.name}] "
                    f"[{self.enterprise_configuration.channel_code()}]. "
                    f"Task failed with message [{response_body}]"
                )
            finally:
                action_happened_at = localized_utcnow()
                for content_id, transmission in chunk.items():
                    transmission.api_response_status_code = response_status_code
                    was_successful = response_status_code < 300
                    api_content_response = response_body
                    if was_successful:
                        api_content_response = self._filter_api_response(api_content_response, content_id)
                    (api_content_response, was_truncated) = truncate_string(api_content_response)
                    if was_truncated:
                        self._log_info(
                            f'integrated_channel_content_transmission_id={transmission.id}, '
                            f'api response truncated',
                            course_or_course_run_key=content_id
                        )
                    if transmission.api_record:
                        transmission.api_record.body = api_content_response
                        transmission.api_record.status_code = response_status_code
                        transmission.api_record.save()
                    else:
                        ApiResponseRecord = apps.get_model(
                            'integrated_channel',
                            'ApiResponseRecord'
                        )
                        transmission.api_record = ApiResponseRecord.objects.create(
                            body=api_content_response, status_code=response_status_code
                        )
                    if action_name == 'create':
                        transmission.remote_created_at = action_happened_at
                    elif action_name == 'update':
                        transmission.remote_updated_at = action_happened_at
                    elif is_delete_action:
                        transmission.remote_deleted_at = action_happened_at
                        if was_successful:
                            successfully_removed_content_keys.append(transmission.content_id)
                    transmission.save()
                    if was_successful:
                        transmission.remove_marked_for()
                    self.enterprise_configuration.update_content_synced_at(action_happened_at, was_successful)
                    results.append(transmission)

        if is_delete_action and successfully_removed_content_keys:
            # Mark any successfully deleted, orphaned content transmissions as resolved
            orphaned_items.filter(content_id__in=successfully_removed_content_keys).update(resolved=True)

        return results



# Generated by Django 3.2.8 on 2021-11-01 20:21

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('degreed2', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='degreed2enterprisecustomerconfiguration',
            old_name='key',
            new_name='client_id',
        ),
        migrations.RenameField(
            model_name='degreed2enterprisecustomerconfiguration',
            old_name='secret',
            new_name='client_secret',
        ),
        migrations.RenameField(
            model_name='historicaldegreed2enterprisecustomerconfiguration',
            old_name='key',
            new_name='client_id',
        ),
        migrations.RenameField(
            model_name='historicaldegreed2enterprisecustomerconfiguration',
            old_name='secret',
            new_name='client_secret',
        ),
    ]



# -*- coding: utf-8 -*-
"""
Client for connecting to Degreed2.
"""

import json
import logging
import time
from http import HTTPStatus

import requests
from six.moves.urllib.parse import urljoin

from django.apps import apps
from django.conf import settings
from django.http.request import QueryDict

from enterprise.models import EnterpriseCustomerUser
from integrated_channels.exceptions import ClientError
from integrated_channels.integrated_channel.client import IntegratedChannelApiClient
from integrated_channels.utils import generate_formatted_log, refresh_session_if_expired

LOGGER = logging.getLogger(__name__)


class Degreed2APIClient(IntegratedChannelApiClient):
    """
    Client for connecting to Degreed2.

    Specifically, this class supports obtaining access tokens and posting to the courses and
    completion status endpoints.
    """

    CONTENT_WRITE_SCOPE = "content:write"
    ALL_DESIRED_SCOPES = "content:read,content:write,completions:write,completions:read"
    SESSION_TIMEOUT = getattr(settings, "ENTERPRISE_DEGREED2_SESSION_TIMEOUT", 60)
    MAX_RETRIES = getattr(settings, "ENTERPRISE_DEGREED2_MAX_RETRIES", 4)
    BACKOFF_FACTOR = getattr(settings, "ENTERPRISE_DEGREED2_BACKOFF_FACTOR", 2)

    def __init__(self, enterprise_configuration):
        """
        Instantiate a new client.

        Args:
            enterprise_configuration (Degreed2EnterpriseCustomerConfiguration): An enterprise customers's
            configuration model for connecting with Degreed
        """
        super().__init__(enterprise_configuration)
        self.session = None
        self.expires_at = None
        app_config = apps.get_app_config('degreed2')
        self.oauth_api_path = app_config.oauth_api_path
        self.courses_api_path = app_config.courses_api_path
        self.completions_api_path = app_config.completions_api_path
        self.skill_api_path = app_config.skill_api_path
        # to log without having to pass channel_name, ent_customer_uuid each time
        self.make_log_msg = lambda course_key, message, lms_user_id=None: generate_formatted_log(
            self.enterprise_configuration.channel_code(),
            self.enterprise_configuration.enterprise_customer.uuid,
            lms_user_id,
            course_key,
            message,
        )

    def get_oauth_url(self):
        config = self.enterprise_configuration
        base_url = config.degreed_token_fetch_base_url or config.degreed_base_url
        return urljoin(base_url, self.oauth_api_path)

    def get_courses_url(self):
        return urljoin(self.enterprise_configuration.degreed_base_url, self.courses_api_path)

    def get_completions_url(self):
        return urljoin(self.enterprise_configuration.degreed_base_url, self.completions_api_path)

    def get_course_skills_url(self, course_key):
        return urljoin(
            self.enterprise_configuration.degreed_base_url,
            self.skill_api_path.format(contentId=course_key)
        )

    def create_assessment_reporting(self, user_id, payload):
        """
        Not implemented yet.
        """
        LOGGER.error(
            generate_formatted_log(
                self.enterprise_configuration.channel_code(),
                self.enterprise_configuration.enterprise_customer.uuid,
                None,
                None,
                "Degreed2 integrated channel does not yet support assessment reporting."
            )
        )

    def cleanup_duplicate_assignment_records(self, courses):
        """
        Not implemented yet.
        """
        LOGGER.error(
            generate_formatted_log(
                self.enterprise_configuration.channel_code(),
                self.enterprise_configuration.enterprise_customer.uuid,
                None,
                None,
                "Degreed2 integrated channel does not yet support assignment deduplication."
            )
        )

    def create_course_completion(self, user_id, payload):
        """
        Completions status for a learner
        Ref: https://api.degreed.com/docs/#create-a-new-completion

        Arguments:
            - user_id: learner's id
            - payload: (dict) with keys:
                 {
                     degreed_user_email,
                     course_id,
                     completed_timestamp, (in the format "2018-08-01T00:00:00")
                 }
        Returns: status_code, response_text
        """
        json_payload = json.loads(payload)
        code, body = self._post(
            self.get_completions_url(),
            json_payload,
            self.ALL_DESIRED_SCOPES
        )
        if code == HTTPStatus.BAD_REQUEST.value:
            error_response = json.loads(body)
            for error in error_response['errors']:
                if 'detail' in error and 'Invalid user identifier' in error['detail']:
                    try:
                        enterprise_customer = self.enterprise_configuration.enterprise_customer
                        LOGGER.info(
                            generate_formatted_log(
                                self.enterprise_configuration.channel_code(),
                                self.enterprise_configuration.enterprise_customer.uuid,
                                None,
                                None,
                                f'User {user_id} was deleted on degreed side,'
                                f"so marking it as inactive and unlinking from enterprise"
                            )
                        )
                        # Unlink user from related Enterprise Customer
                        EnterpriseCustomerUser.objects.unlink_user(
                            enterprise_customer=enterprise_customer,
                            user_email=json_payload.get('data').get('attributes').get('user-id'),
                        )
                    except Exception as e:  # pylint: disable=broad-except
                        LOGGER.error(
                            generate_formatted_log(
                                self.enterprise_configuration.channel_code(),
                                self.enterprise_configuration.enterprise_customer.uuid,
                                None,
                                None,
                                f'Error occurred while unlinking a degreed2 learner: {user_id}. '
                                f'Payload: {json_payload}, Error: {e}'
                            )
                        )
        return code, body

    def delete_course_completion(self, user_id, payload):
        """
        Not implemented yet. deletion requires completion ID. We don't have a way to get the ID right now.
        So we may need to store id on our side to be able to do this.
        https://api.degreed.com/docs/#delete-a-specific-completion
        """
        LOGGER.error(
            generate_formatted_log(
                self.enterprise_configuration.channel_code(),
                self.enterprise_configuration.enterprise_customer.uuid,
                None,
                None,
                "Degreed2 integrated channel does not yet support deleting course completions."
            )
        )

    def fetch_degreed_course_id(self, external_id):
        """
        Fetch the 'id' of a course from Degreed2, given the external-id as a search param
        'external-id' is the edX course key
        """
        # QueryDict converts + to space
        params = QueryDict(f"filter[external_id]={external_id.replace('+','%2B')}")
        course_search_url = f'{self.get_courses_url()}?{params.urlencode(safe="[]")}'
        LOGGER.info(self.make_log_msg(external_id, f'Attempting find course via url: {course_search_url}'))
        status_code, response_body = self._get(
            course_search_url,
            self.ALL_DESIRED_SCOPES
        )
        if status_code >= 400:
            raise ClientError(
                f'Degreed2: fetch request failed: attempted, external ID={external_id},'
                f'received status_code={status_code}',
                status_code=status_code
            )
        response_json = json.loads(response_body)
        if response_json['data']:
            return response_json['data'][0]['id']
        raise ClientError(
            f'Degreed2: Attempted to find degreed course id but failed, external id was {external_id}'
            f', Response from Degreed was {response_body}')

    def assign_course_skills(self, course_id, serialized_data):
        """
        Assign skills to a course.

        Args:
            serialized_data: JSON-encoded object containing skills metadata.

        Raises:
            ClientError:
                If degreed course id doesn't exist.
                If Degreed course skills API request fails.
        """

        degreed_course_id = self.fetch_degreed_course_id(course_id)
        if not degreed_course_id:
            raise ClientError(f'Degreed2: Cannot find course via external-id {course_id}')

        course_skills_url = self.get_course_skills_url(degreed_course_id)
        LOGGER.info(self.make_log_msg(course_id, f'Attempting to assign course skills {course_skills_url}'))
        try:
            status_code, response_body = self._patch(course_skills_url, serialized_data, self.CONTENT_WRITE_SCOPE)
            if status_code == 201:
                LOGGER.info(
                    self.make_log_msg(
                        course_id,
                        f'Succesfully assigned skills to course {course_id}')
                )
            elif status_code >= 400:
                raise ClientError(
                    f'Degreed2APIClient failed to assign skills to course {course_id}.'
                    f'Failed with status_code={status_code} and response={response_body}',
                )
        except requests.exceptions.RequestException as exc:
            raise ClientError(
                'Degreed2APIClient request to assign skills failed: {error} {message}'.format(
                    error=exc.__class__.__name__,
                    message=str(exc)
                )
            ) from exc

    def create_content_metadata(self, serialized_data):
        """
        Create content metadata using the Degreed course content API.

        Args:
            serialized_data: JSON-encoded object containing content metadata.

        Raises:
            ClientError: If Degreed API request fails.
        """
        channel_metadata_item = json.loads(serialized_data.decode('utf-8'))
        # only expect one course in this array as of now (chunk size is 1)
        a_course = channel_metadata_item['courses'][0]
        status_code, response_body = self._sync_content_metadata(a_course, 'post', self.get_courses_url())
        if status_code == 409:
            # course already exists, don't raise failure, but log and move on
            LOGGER.warning(
                self.make_log_msg(
                    a_course.get('external-id'),
                    f'Course with integration_id = {a_course.get("external-id")} already exists, '
                )
            )
            # content already exists, we'll treat this as a success
            status_code = 200
        elif status_code >= 400:
            raise ClientError(
                f'Degreed2APIClient create_content_metadata failed with status {status_code}: {response_body}',
                status_code=status_code
            )
        return status_code, response_body

    def update_content_metadata(self, serialized_data):
        """
        Update content metadata using the Degreed course content API.

        Args:
            serialized_data: JSON-encoded object containing content metadata.

        Raises:
            ClientError: If Degreed API request fails.
        """
        channel_metadata_item = json.loads(serialized_data.decode('utf-8'))
        course_item = channel_metadata_item['courses'][0]
        external_id = course_item.get('external-id')

        course_id = self.fetch_degreed_course_id(external_id)
        if not course_id:
            raise ClientError(f'Degreed2: Cannot find course via external-id {external_id}')

        patch_url = f'{self.get_courses_url()}/{course_id}'
        LOGGER.info(self.make_log_msg(external_id, f'Attempting course update via {patch_url}'))
        patch_status_code, patch_response_body = self._sync_content_metadata(
            course_item,
            'patch',
            patch_url,
            course_id
        )
        return patch_status_code, patch_response_body

    def delete_content_metadata(self, serialized_data):
        """
        Delete content metadata using the Degreed course content API.

        Args:
            serialized_data: JSON-encoded object containing content metadata.

        Raises:
            ClientError: If Degreed API request fails.
        """
        # {{base_api_url}}/api/v2/content/courses?filter[external_id]=course-v1%3AedX%2B444555666%2B3T2021
        channel_metadata_item = json.loads(serialized_data.decode('utf-8'))
        course_item = channel_metadata_item['courses'][0]
        external_id = course_item.get('external-id')

        del_status_code = None
        del_response_body = ''
        try:
            course_id = self.fetch_degreed_course_id(external_id)
            if not course_id:
                raise ClientError(f'Degreed2: Cannot find course via external-id {external_id}')

            del_url = f'{self.get_courses_url()}/{course_id}'
            LOGGER.info(self.make_log_msg(external_id, f'Attempting course delete via {del_url}'))
            del_status_code, del_response_body = self._delete(
                del_url,
                None,
                self.ALL_DESIRED_SCOPES
            )
            if del_status_code >= 400:
                raise ClientError(
                    f'Degreed2: delete request failed: attempted, external ID={external_id},'
                    f'status_code={del_status_code}'
                    f'response_body={del_response_body}',
                    status_code=del_status_code
                )
        except requests.exceptions.RequestException as exc:
            raise ClientError(
                'Degreed2APIClient delete request failed: {error} {message}'.format(
                    error=exc.__class__.__name__,
                    message=str(exc)
                )
            ) from exc

        return del_status_code, del_response_body

    def _sync_content_metadata(self, course_attributes, http_method, override_url, degreed_course_id=None):
        """
        Synchronize content metadata using the Degreed course content API.
        Invokes the Create/Update operations of degreed api
        Used for create course, and update course use cases
        The json sent contains

        Args:
            course_attributes: JSON object containing content metadata converted into
              Degreed2 'attributes' in the payload.
            http_method: The HTTP method to use for the API request.
            override_url: uses this url to post to
            course_id: used to append at the end of the url as /ID if provided

        Raises:
            ClientError: If Degreed API request fails.
        """
        json_to_send = {
            "data": {
                "type": "content/courses",
                "attributes": course_attributes,
            }
        }

        # useful for update use case
        if degreed_course_id:
            json_to_send['data']['id'] = degreed_course_id

        try:
            status_code, response_body = getattr(self, '_' + http_method)(
                override_url,
                json_to_send,
                self.ALL_DESIRED_SCOPES
            )
        except requests.exceptions.RequestException as exc:
            raise ClientError(
                'Degreed2APIClient request failed: {error} {message}'.format(
                    error=exc.__class__.__name__,
                    message=str(exc)
                )
            ) from exc
        return status_code, response_body

    def _calculate_backoff(self, attempt_count):
        """
        Calculate the seconds to sleep based on attempt_count
        """
        return (self.BACKOFF_FACTOR * (2 ** (attempt_count - 1)))

    def _get(self, url, scope):
        """
        Make a GET request using the session object to a Degreed2 endpoint.

        Args:
            url (str): The url to send a GET request to.
            scope (str): Must be one of the scopes Degreed expects:
                        - `CONTENT_WRITE_SCOPE`
                        - `CONTENT_READ_SCOPE`
        """
        self._create_session(scope)
        attempts = 0
        while True:
            attempts = attempts + 1
            response = self.session.get(url)
            if attempts <= self.MAX_RETRIES and response.status_code == 429:
                sleep_seconds = self._calculate_backoff(attempts)
                LOGGER.warning(
                    generate_formatted_log(
                        self.enterprise_configuration.channel_code(),
                        self.enterprise_configuration.enterprise_customer.uuid,
                        None,
                        None,
                        f'[Degreed2Client]._get 429 detected from {url}, backing-off before retrying, '
                        f'sleeping {sleep_seconds} seconds...'
                    )
                )
                time.sleep(sleep_seconds)
            else:
                LOGGER.error(
                    generate_formatted_log(
                        self.enterprise_configuration.channel_code(),
                        self.enterprise_configuration.enterprise_customer.uuid,
                        None,
                        None,
                        '[Degreed2Client]._get - Exceeded retry attempts in:'
                        f'URL:{url}'
                    )
                )
                break
        return response.status_code, response.text

    def _post(self, url, data, scope):
        """
        Make a POST request using the session object to a Degreed2 endpoint.

        Args:
            url (str): The url to send a POST request to.
            data (dict): The json payload to POST.
            scope (str): Must be one of the scopes Degreed expects:
                        - `CONTENT_WRITE_SCOPE`
                        - `CONTENT_READ_SCOPE`
        """
        self._create_session(scope)
        attempts = 0
        while True:
            attempts = attempts + 1
            response = self.session.post(url, json=data)
            if attempts <= self.MAX_RETRIES and response.status_code == 429:
                sleep_seconds = self._calculate_backoff(attempts)
                LOGGER.warning(
                    generate_formatted_log(
                        self.enterprise_configuration.channel_code(),
                        self.enterprise_configuration.enterprise_customer.uuid,
                        None,
                        None,
                        f'[Degreed2Client]._post 429 detected from {url}, backing-off before retrying, '
                        f'sleeping {sleep_seconds} seconds...'
                    )
                )
                time.sleep(sleep_seconds)
            else:
                LOGGER.error(
                    generate_formatted_log(
                        self.enterprise_configuration.channel_code(),
                        self.enterprise_configuration.enterprise_customer.uuid,
                        None,
                        None,
                        '[Degreed2Client]._post - Exceeded retry attempts in:'
                        f'URL:{url}, DATA:{data}'
                    )
                )
                break
        return response.status_code, response.text

    def _patch(self, url, data, scope):
        """
        Make a PATCH request using the session object to a Degreed2 endpoint.

        Args:
            url (str): The url to send a POST request to.
            data (str): The json payload to POST.
            scope (str): Must be one of the scopes Degreed expects:
                        - `CONTENT_WRITE_SCOPE`
                        - `CONTENT_READ_SCOPE`
        """
        self._create_session(scope)
        attempts = 0
        while True:
            attempts = attempts + 1
            response = self.session.patch(url, json=data)
            if attempts <= self.MAX_RETRIES and response.status_code == 429:
                sleep_seconds = self._calculate_backoff(attempts)
                LOGGER.warning(
                    generate_formatted_log(
                        self.enterprise_configuration.channel_code(),
                        self.enterprise_configuration.enterprise_customer.uuid,
                        None,
                        None,
                        f'429 detected from {url}, backing-off before retrying, '
                        f'sleeping {sleep_seconds} seconds...'
                    )
                )
                time.sleep(sleep_seconds)
            else:
                break
        return response.status_code, response.text

    def _delete(self, url, data, scope):
        """
        Make a DELETE request using the session object to a Degreed endpoint.

        Args:
            url (str): The url to send a DELETE request to.
            data (str): The json payload to DELETE. None means nothing is sent in payload.
            scope (str): Must be one of the scopes Degreed expects:
                        - `CONTENT_PROVIDER_SCOPE`
                        - `COMPLETION_PROVIDER_SCOPE`
        """
        self._create_session(scope)
        attempts = 0
        while True:
            attempts = attempts + 1
            response = self.session.delete(url, json=data) if data else self.session.delete(url)
            if attempts <= self.MAX_RETRIES and response.status_code == 429:
                sleep_seconds = self._calculate_backoff(attempts)
                LOGGER.warning(
                    generate_formatted_log(
                        self.enterprise_configuration.channel_code(),
                        self.enterprise_configuration.enterprise_customer.uuid,
                        None,
                        None,
                        f'429 detected from {url}, backing-off before retrying, '
                        f'sleeping {sleep_seconds} seconds...'
                    )
                )
                time.sleep(sleep_seconds)
            else:
                break
        return response.status_code, response.text

    def _create_session(self, scope):
        """
        Instantiate a new session object for use in connecting with Degreed
        """
        self.session, self.expires_at = refresh_session_if_expired(
            lambda: self._get_oauth_access_token(scope),
            self.session,
            self.expires_at,
        )

    def _get_oauth_access_token(self, scope):
        """ Retrieves OAuth 2.0 access token using the client credentials grant.
        Prefers using the degreed_token_fetch_base_url over the degreed_base_url, if present, to fetch the access token.

        Args:
            scope (str): Must be one or comma separated list of the scopes Degreed expects
        Returns:
            tuple: Tuple containing access token string and expiration datetime.
        Raises:
            HTTPError: If we received a failure response code from Degreed.
            ClientError: If an unexpected response format was received that we could not parse.
        """
        config = self.enterprise_configuration
        response = requests.post(
            self.get_oauth_url(),
            data={
                'grant_type': 'client_credentials',
                'scope': scope,
                'client_id': config.client_id,
                'client_secret': config.client_secret,
            },
            headers={'Content-Type': 'application/x-www-form-urlencoded'}
        )

        try:
            data = response.json()
            return data['access_token'], data['expires_in']
        except (KeyError, ValueError) as error:
            raise ClientError(response.text, response.status_code) from error



"""
API clients to communicate with Braze services.
"""
import logging

from braze.client import BrazeClient

from django.conf import settings

logger = logging.getLogger(__name__)


class BrazeAPIClient:
    """
    API client for calls to Braze.
    """
    @classmethod
    def get_braze_client(cls):
        """ Returns a Braze client. """
        if not BrazeClient:
            return None

        # fetching them from edx-platform settings
        braze_api_key = getattr(settings, 'EDX_BRAZE_API_KEY', None)
        braze_api_url = getattr(settings, 'EDX_BRAZE_API_SERVER', None)

        if not braze_api_key or not braze_api_url:
            return None

        return BrazeClient(
            api_key=braze_api_key,
            api_url=braze_api_url,
            app_id='',
        )



# Generated by Django 3.2.20 on 2023-11-06 12:33

from django.db import migrations


def populate_decrypted_fields(apps, schema_editor):
    """
    Populates the encryption fields with the data previously stored in database.
    """
    MoodleEnterpriseCustomerConfiguration = apps.get_model('moodle', 'MoodleEnterpriseCustomerConfiguration')

    for moodle_enterprise_configuration in MoodleEnterpriseCustomerConfiguration.objects.all():
        moodle_enterprise_configuration.decrypted_username = moodle_enterprise_configuration.username
        moodle_enterprise_configuration.decrypted_password = moodle_enterprise_configuration.password
        moodle_enterprise_configuration.decrypted_token = moodle_enterprise_configuration.token
        moodle_enterprise_configuration.save()


class Migration(migrations.Migration):

    dependencies = [
        ('moodle', '0028_auto_20230928_1530'),
    ]

    operations = [
        migrations.RunPython(populate_decrypted_fields, reverse_code=migrations.RunPython.noop),
    ]



"""
Utility functions for xAPI.
"""

import logging

from enterprise.tpa_pipeline import get_user_social_auth
from integrated_channels.exceptions import ClientError
from integrated_channels.xapi.client import EnterpriseXAPIClient
from integrated_channels.xapi.statements.learner_course_completion import LearnerCourseCompletionStatement
from integrated_channels.xapi.statements.learner_course_enrollment import LearnerCourseEnrollmentStatement

LOGGER = logging.getLogger(__name__)


def _send_statement(statement, object_type, event_type, lrs_configuration,
                    customer_name, username, course_id, response_fields):
    """
    Transmit the specified xAPI Event information to the specified xAPI Learning Record Store service.
    """

    LOGGER.info(
        '[Integrated Channel][xAPI] Sending {object_type} enrollment to xAPI LRS for user: {username} '
        'for {object_type}: {course_id}'.format(
            object_type=object_type,
            username=username,
            course_id=course_id,
        )
    )

    lrs_client = EnterpriseXAPIClient(lrs_configuration)

    try:
        lrs_response = lrs_client.save_statement(statement)
        response_fields.update({
            'status': lrs_response.response.status,
            'error_message': lrs_response.data
        })

    except ClientError:
        error_message = 'EnterpriseXAPIClient request failed.'
        response_fields.update({
            'error_message': error_message
        })
        LOGGER.exception(error_message)

    status_string = 'Error transmitting'
    if response_fields['status'] == 200:
        status_string = 'Successfully transmitted'

    LOGGER.info(
        '[Integrated Channel][xAPI] {status_string} {object_type} {event_type} event to {lrs_hostname} for '
        'Enterprise Customer: {enterprise_customer}, User: {username} '
        'and {object_type}: {course_id}'.format(
            status_string=status_string,
            object_type=object_type,
            event_type=event_type,
            lrs_hostname=lrs_configuration.endpoint,
            enterprise_customer=customer_name,
            username=username,
            course_id=course_id,
        )
    )

    return response_fields


def send_course_enrollment_statement(lrs_configuration, user, course_overview, object_type, response_fields):
    """
    Send xAPI statement for course enrollment.

    Arguments:
         lrs_configuration (XAPILRSConfiguration): XAPILRSConfiguration instance where to send statements.
         user (User): User object.
         course_overview (CourseOverview): CourseOverview object containing course details.
    """
    event_type = 'enrollment'
    course_id = course_overview.course_key if object_type == 'course' else str(course_overview.id)
    username = user.username if user else 'Unavailable'
    user_social_auth = get_user_social_auth(user, lrs_configuration.enterprise_customer)

    statement = LearnerCourseEnrollmentStatement(
        lrs_configuration.enterprise_customer.site,
        user,
        user_social_auth,
        course_overview,
        object_type,
    )

    response_fields = _send_statement(
        statement,
        object_type,
        event_type,
        lrs_configuration,
        lrs_configuration.enterprise_customer.name,
        username,
        course_id,
        response_fields,
    )

    return response_fields


def send_course_completion_statement(lrs_configuration,
                                     user, course_overview, course_grade, object_type, response_fields):
    """
    Send xAPI statement for course completion.

    Arguments:
         lrs_configuration (XAPILRSConfiguration): XAPILRSConfiguration instance where to send statements.
         user (User): User object.
         course_overview (CourseOverview): Course overview object containing course details.
         course_grade (CourseGrade): Course grade object.
    """
    event_type = 'completion'
    course_id = course_overview.course_key if object_type == 'course' else str(course_overview.id)
    username = user.username if user else 'Unavailable'
    user_social_auth = get_user_social_auth(user, lrs_configuration.enterprise_customer)

    statement = LearnerCourseCompletionStatement(
        lrs_configuration.enterprise_customer.site,
        user,
        user_social_auth,
        course_overview,
        course_grade,
        object_type,
    )

    response_fields = _send_statement(
        statement,
        object_type,
        event_type,
        lrs_configuration,
        lrs_configuration.enterprise_customer.name,
        username,
        course_id,
        response_fields,
    )

    return response_fields


def is_success_response(response_fields):
    """
    Determines if a response is successful or not based on captured information
    Arguments: response_fields (dict)
    Returns: Boolean
    """
    success_response = False
    if response_fields['status'] == 200:
        success_response = True
    return success_response



"""
Enterprise Django application constants.
"""
from django.utils.translation import gettext_lazy as _

# We listen to the User post_save signal in order to associate new users
# with an EnterpriseCustomer when applicable. This it the unique identifier
# used to ensure that signal receiver is only called once.
USER_POST_SAVE_DISPATCH_UID = "user_post_save_upgrade_pending_enterprise_customer_user"

# Data sharing consent messages
CONSENT_REQUEST_PROMPT = _(
    'To log in using this SSO identity provider and access special course offers, you must first '
    'consent to share your learning achievements with {enterprise_customer_name}.'
)
CONFIRMATION_ALERT_PROMPT = _(
    'In order to sign in and access special offers, you must consent to share your '
    'course data with {enterprise_customer_name}.'
)
CONFIRMATION_ALERT_PROMPT_WARNING = _(
    'If you do not consent to share your course data, that information may be shared with '
    '{enterprise_customer_name}.'
)
WELCOME_TEXT = _('Welcome to {platform_name}.')
ENTERPRISE_WELCOME_TEXT = _(
    'You have left the {strong_start}{enterprise_customer_name}{strong_end} website and are now on the '
    '{platform_name} site. {enterprise_customer_name} has partnered with {platform_name} to offer you '
    'high-quality, always available learning programs to help you advance your knowledge and career. '
    '{line_break}Please note that {platform_name} has a different {privacy_policy_link_start}Privacy Policy '
    '{privacy_policy_link_end} from {enterprise_customer_name}.'
)

COURSE_KEY_URL_PATTERN = r'(?P<course_key>[^/+]+(/|\+)[^/+]+)'

MAX_INVITE_KEYS = 100


class DefaultColors:
    """
    Class to group the default branding color codes.
    These color codes originated in the Enterprise Learner Portal.
    """
    PRIMARY = '#2D494E'
    SECONDARY = '#F2F0EF'
    TERTIARY = '#D23228'


class CourseModes:
    """
    Class to group modes that a course might have.
    """

    AUDIT = 'audit'
    CREDIT = 'credit'
    HONOR = 'honor'
    NO_ID_PROFESSIONAL = 'no-id-professional'
    PROFESSIONAL = 'professional'
    VERIFIED = 'verified'
    UNPAID_EXECUTIVE_EDUCATION = 'unpaid-executive-education'


BEST_MODE_ORDER = [
    CourseModes.VERIFIED,
    CourseModes.PROFESSIONAL,
    CourseModes.NO_ID_PROFESSIONAL,
    CourseModes.UNPAID_EXECUTIVE_EDUCATION,
]

# Course mode sorting based on slug
COURSE_MODE_SORT_ORDER = [
    CourseModes.VERIFIED,
    CourseModes.PROFESSIONAL,
    CourseModes.NO_ID_PROFESSIONAL,
    CourseModes.AUDIT,
    CourseModes.HONOR,
    CourseModes.UNPAID_EXECUTIVE_EDUCATION,
]

EXEC_ED_COURSE_TYPE = "executive-education-2u"
PRODUCT_SOURCE_2U = "2u"
EXEC_ED_CONTENT_DESCRIPTION_TAG = ("This instructor-led Executive Education course is "
                                   "presented by GetSmarter, an edX partner. ")

# Course modes that should not be displayed to users.
EXCLUDED_COURSE_MODES = [CourseModes.CREDIT]

# Number of records to display in each paginated set.
PAGE_SIZE = 25

PROGRAM_TYPE_DESCRIPTION = {
    'MicroMasters Certificate': _(
        'A series of Master’s-level courses to advance your career, '
        'created by top universities and recognized by companies. '
        'MicroMasters Programs are credit-eligible, provide in-demand '
        'knowledge and may be applied to accelerate a Master’s Degree.'
    ),
    'Professional Certificate': _(
        'Designed by industry leaders and top universities to enhance '
        'professional skills, Professional Certificates develop the '
        'proficiency and expertise that employers are looking for with '
        'specialized training and professional education.'
    ),
    'XSeries Certificate': _(
        'Created by world-renowned experts and top universities, XSeries '
        'are designed to provide a deep understanding of key subjects '
        'through a series of courses. Complete the series to earn a valuable '
        'XSeries Certificate that illustrates your achievement.'
    ),
}

ALLOWED_TAGS = [
    'a', 'abbr', 'acronym', 'b', 'blockquote', 'em', 'i',
    'li', 'ol', 'strong', 'ul', 'p', 'h1', 'h2',
]

DEFAULT_CATALOG_CONTENT_FILTER = {
    'content_type': 'course',
    'partner': 'edx',
    'level_type': [
        'Introductory',
        'Intermediate',
        'Advanced'
    ],
    'availability': [
        'Current',
        'Starting Soon',
        'Upcoming'
    ],
    'status': 'published'
}

# Django groups specific to granting permission to enterprise admins.
ENTERPRISE_DATA_API_ACCESS_GROUP = 'enterprise_data_api_access'
ENTERPRISE_ENROLLMENT_API_ACCESS_GROUP = 'enterprise_enrollment_api_access'
ENTERPRISE_PERMISSION_GROUPS = [
    ENTERPRISE_ENROLLMENT_API_ACCESS_GROUP,
    ENTERPRISE_DATA_API_ACCESS_GROUP,
]

ENTERPRISE_LEARNER_ROLE = 'enterprise_learner'
ENTERPRISE_ADMIN_ROLE = 'enterprise_admin'
ENTERPRISE_OPERATOR_ROLE = 'enterprise_openedx_operator'
SYSTEM_ENTERPRISE_CATALOG_ADMIN_ROLE = 'enterprise_catalog_admin'

ENTERPRISE_DASHBOARD_ADMIN_ROLE = 'dashboard_admin'
ENTERPRISE_CATALOG_ADMIN_ROLE = 'catalog_admin'
ENTERPRISE_ENROLLMENT_API_ADMIN_ROLE = 'enrollment_api_admin'
ENTERPRISE_REPORTING_CONFIG_ADMIN_ROLE = 'reporting_config_admin'
ENTERPRISE_FULFILLMENT_OPERATOR_ROLE = 'fulfillment_operator'
ENTERPRISE_SSO_ORCHESTRATOR_OPERATOR_ROLE = 'sso_orchestrator_operator'

# Tracking related
PATHWAY_CUSTOMER_ADMIN_ENROLLMENT = 'customer-admin-enrollment'

# context to give access to all resources
ALL_ACCESS_CONTEXT = '*'

# this constant should not be used anymore as it is marked for removal
ENTERPRISE_ROLE_BASED_ACCESS_CONTROL_SWITCH = 'enterprise_role_based_access_control'

OAUTH2_PROVIDER_APPLICATION_MODEL = 'oauth2_provider.Application'

EDX_ORG_NAME = 'edX, Inc'

# Waffle flag used to switch over edx-enterprise's usage of the enterprise catalog service
USE_ENTERPRISE_CATALOG = 'use_enterprise_catalog'

# ContentFilter field types for validation.
CONTENT_FILTER_FIELD_TYPES = {
    'key': {'type': list, 'subtype': str},
    'first_enrollable_paid_seat_price__lte': {'type': str}
}

# FEATURE flag that indicates if deletion of EnterpriseCourseEnrollment records via
# the Django Admin site is allowed.
ALLOW_ADMIN_ENTERPRISE_COURSE_ENROLLMENT_DELETION = 'ALLOW_ADMIN_ENTERPRISE_COURSE_ENROLLMENT_DELETION'


def json_serialized_course_modes():
    """
    :return: serialized course modes.
    """
    return COURSE_MODE_SORT_ORDER


AVAILABLE_LANGUAGES = [
    ('en', 'English'),
    ('es-419', 'Español (Latinoamérica)'),  # Spanish (Latin America)
]

LMS_API_DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%SZ'
LMS_API_DATETIME_FORMAT_WITHOUT_TIMEZONE = '%Y-%m-%dT%H:%M:%S'

TRANSMISSION_MARK_CREATE = 'create'
TRANSMISSION_MARK_UPDATE = 'update'
TRANSMISSION_MARK_DELETE = 'delete'

HTTP_STATUS_STRINGS = {
    400: 'The request was invalid, check the fields you entered are correct.',
    401: 'The request was unauthorized, check your credentials.',
    403: 'The request was rejected because it did not have the rights to access the content.',
    404: 'The requested resource was not found.',
    408: 'The request timed out.',
    429: 'The user has sent too many requests.',
    500: 'An internal problem on our side interfered.',
    503: 'The server is temporarily unavailable.',
}

IC_CREATE_ACTION = 'create'
IC_UPDATE_ACTION = 'update'
IC_DELETE_ACTION = 'delete'


class FulfillmentTypes:
    LICENSE = 'license'
    LEARNER_CREDIT = 'learner_credit'
    COUPON_CODE = 'coupon_code'
    CHOICES = [(choice, choice.capitalize().replace('_', ' ')) for choice in (LICENSE, LEARNER_CREDIT, COUPON_CODE)]


SSO_BRAZE_CAMPAIGN_ID = 'a5f10d46-8093-4ce1-bab7-6df018d03660'

# The maximum length of a text field in the database.
MAX_ALLOWED_TEXT_LENGTH = 16_000_000



# Generated by Django 1.11.23 on 2019-09-25 07:30


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cornerstone', '0004_cornerstoneglobalconfiguration_languages'),
    ]

    operations = [
        migrations.AddField(
            model_name='cornerstoneenterprisecustomerconfiguration',
            name='channel_worker_username',
            field=models.CharField(blank=True, help_text='Enterprise channel worker username to get JWT tokens for authenticating LMS APIs.', max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='historicalcornerstoneenterprisecustomerconfiguration',
            name='channel_worker_username',
            field=models.CharField(blank=True, help_text='Enterprise channel worker username to get JWT tokens for authenticating LMS APIs.', max_length=255, null=True),
        ),
    ]



"""
Throttle classes for enterprise API.
"""

from rest_framework.throttling import UserRateThrottle

from enterprise.api.utils import get_service_usernames

SERVICE_USER_SCOPE = 'service_user'
HIGH_SERVICE_USER_SCOPE = 'high_service_user'


class BaseThrottle(UserRateThrottle):
    """
    Base throttle class with common functionality.
    """

    def allow_request(self, request, view):
        """
        Modify throttling for service users.

        Updates throttling rate if the request is coming from the service user, and defaults to UserRateThrottle's
        configured setting otherwise.

        Updated throttling rate comes from `DEFAULT_THROTTLE_RATES` key in `REST_FRAMEWORK` setting. specific user
        throttling is specified in `DEFAULT_THROTTLE_RATES` by it's corresponding key.

        .. code-block::

            REST_FRAMEWORK = {
                'DEFAULT_THROTTLE_RATES': {
                    'service_user': '50/day',
                    'high_service_user': '2000/minute',
                }
            }
        """
        service_users = get_service_usernames()

        # Update throttle scope if the request user is a service user.
        if request.user.username in service_users:
            self.update_throttle_scope()

        return super().allow_request(request, view)

    def get_scope(self):
        """
        Get the scope of the throttle.

        Raises:
            NotImplementedError: Must be implemented by subclasses
        """
        raise NotImplementedError("Subclasses must implement get_scope.")

    def update_throttle_scope(self):
        """
        Update throttle scope based on the specific subclass.
        """
        self.scope = self.get_scope()
        self.rate = self.get_rate()
        self.num_requests, self.duration = self.parse_rate(self.rate)


class ServiceUserThrottle(BaseThrottle):
    """
    A throttle allowing the service user to override rate limiting.
    """

    def get_scope(self):
        """
        Get the scope of the throttle.

        Returns:
            str: The scope of the throttle.
        """
        return SERVICE_USER_SCOPE


class HighServiceUserThrottle(BaseThrottle):
    """
    A throttle for high service users.
    """

    def get_scope(self):
        """
        Get the scope of the throttle.

        Returns:
            str: The scope of the throttle.
        """
        return HIGH_SERVICE_USER_SCOPE



# Generated by Django 3.2.19 on 2023-06-29 23:30

from django.db import migrations, models
import django.db.models.deletion
import enterprise.models


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('enterprise', '0174_auto_20230608_2041'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='active',
            field=models.BooleanField(default=True, verbose_name='Active admin portal'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='auth_org_id',
            field=models.CharField(blank=True, help_text="Enterprise customer's authentication ID if leveraging a third party platform such as Auth0 for authentication.", max_length=80, null=True, verbose_name='Third Party Auth Org ID:'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='contact_email',
            field=models.EmailField(blank=True, help_text='Email address presented on learner portal as public point of contact from customer organization.', max_length=254, null=True, verbose_name='Customer admin contact email:'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='customer_type',
            field=models.ForeignKey(default=enterprise.models.get_default_customer_type, on_delete=django.db.models.deletion.CASCADE, to='enterprise.enterprisecustomertype', verbose_name='Customer Type'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='default_language',
            field=models.CharField(blank=True, choices=[('en', 'English'), ('es-419', 'Español (Latinoamérica)')], default=None, help_text='Specifies the default language for learners of the organization.', max_length=25, null=True, verbose_name='Learner default language'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_analytics_screen',
            field=models.BooleanField(default=True, help_text='Automatically enabled. Displays advanced analytics page on the administrator portal, which includes skill and labor market data.', verbose_name='Display analytics page'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_audit_data_reporting',
            field=models.BooleanField(default=False, help_text='Enables transmission of audit enrollment data from learning platform learners.', verbose_name='Enable audit enrollment data reporting for learning platform learners'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_audit_enrollment',
            field=models.BooleanField(default=False, help_text='Allows learners enrolling through learning platforms to select the audit track.', verbose_name='Enable audit enrollment for learning platform learners'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_browse_and_request',
            field=models.BooleanField(default=False, verbose_name='Display browse and request management settings'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_data_sharing_consent',
            field=models.BooleanField(default=False, help_text='Enables data sharing consent prompt for learners each time they enroll in a course. If left unchecked, the prompt will not appear and relevant data will not be shared.', verbose_name='Activate data sharing consent prompt'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_integrated_customer_learner_portal_search',
            field=models.BooleanField(default=True, help_text="Automatically enabled. If unchecked, the learners won't be able to search for a course on the learner portal.", verbose_name='Allow course discovery within the learner portal'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_learner_portal',
            field=models.BooleanField(default=True, help_text="Automatically enabled. If unchecked, learners won't have access to the learner portal."),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_learner_portal_offers',
            field=models.BooleanField(default=False, help_text='Specifies whether enterprise offers will be made known to learners in the learner portal This only applies to customers with “offers”, the old version of learner credit.', verbose_name='Enable learner credit in the learner portal'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_portal_code_management_screen',
            field=models.BooleanField(default=False, verbose_name='Display code management screen'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_portal_learner_credit_management_screen',
            field=models.BooleanField(default=False, verbose_name='Display learner credit management screen'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_portal_lms_configurations_screen',
            field=models.BooleanField(default=False, help_text='Enables the learning platform configuration screen on the administrator portal.', verbose_name='Display learning platform configuration screen'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_portal_reporting_config_screen',
            field=models.BooleanField(default=False, help_text='Enables the scheduled reporting configurations screen on the administrator portal.', verbose_name='Display enterprise reporting page'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_portal_saml_configuration_screen',
            field=models.BooleanField(default=False, help_text='Enables the Single Sign On (SSO) configuration screen on the administrator portal. ', verbose_name='Display SSO configuration screen'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_portal_subscription_management_screen',
            field=models.BooleanField(default=False, verbose_name='Display subscription management screen'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_slug_login',
            field=models.BooleanField(default=False, help_text='Allows a learner to input customer slug to identify their org in the SSO process. Should be enabled for customers that leverage SSO.', verbose_name='Allow slug login for SSO'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_universal_link',
            field=models.BooleanField(default=False, verbose_name='Display universal link settings'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enforce_data_sharing_consent',
            field=models.CharField(choices=[('at_enrollment', 'At Enrollment'), ('externally_managed', 'Managed externally')], default='at_enrollment', help_text='Setting to either require learners to accept data sharing consent at course enrollment, or through an external process.', max_length=25, verbose_name='Data sharing consent enforcement:'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='hide_course_original_price',
            field=models.BooleanField(default=False, help_text='Hides course price on learning platform course confirmation screen.', verbose_name='Hide course price on learning platform'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='hide_labor_market_data',
            field=models.BooleanField(default=False, help_text='Hides labor market data from learners (populated by features using Lightcast integration). ', verbose_name='Hide labor market data on skill features'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='replace_sensitive_sso_username',
            field=models.BooleanField(default=False, help_text='Specifies whether to replace the display of potentially sensitive SSO usernames with a more generic name, e.g. EnterpriseLearner.', verbose_name='Replace sensitive SSO username'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='reply_to',
            field=models.EmailField(blank=True, help_text='Email address that will receive learner replies to automated edX emails.', max_length=254, null=True, verbose_name='Customer “reply to” email:'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='sender_alias',
            field=models.CharField(blank=True, help_text='Specifies the sender alias for automated emails from the edX system.', max_length=255, null=True, verbose_name='Automated email sender alias'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='site',
            field=models.ForeignKey(default=enterprise.models.get_default_site, on_delete=django.db.models.deletion.CASCADE, related_name='enterprise_customers', to='sites.site'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='active',
            field=models.BooleanField(default=True, verbose_name='Active admin portal'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='auth_org_id',
            field=models.CharField(blank=True, help_text="Enterprise customer's authentication ID if leveraging a third party platform such as Auth0 for authentication.", max_length=80, null=True, verbose_name='Third Party Auth Org ID:'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='contact_email',
            field=models.EmailField(blank=True, help_text='Email address presented on learner portal as public point of contact from customer organization.', max_length=254, null=True, verbose_name='Customer admin contact email:'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='customer_type',
            field=models.ForeignKey(blank=True, db_constraint=False, default=enterprise.models.get_default_customer_type, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='enterprise.enterprisecustomertype', verbose_name='Customer Type'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='default_language',
            field=models.CharField(blank=True, choices=[('en', 'English'), ('es-419', 'Español (Latinoamérica)')], default=None, help_text='Specifies the default language for learners of the organization.', max_length=25, null=True, verbose_name='Learner default language'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_analytics_screen',
            field=models.BooleanField(default=True, help_text='Automatically enabled. Displays advanced analytics page on the administrator portal, which includes skill and labor market data.', verbose_name='Display analytics page'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_audit_data_reporting',
            field=models.BooleanField(default=False, help_text='Enables transmission of audit enrollment data from learning platform learners.', verbose_name='Enable audit enrollment data reporting for learning platform learners'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_audit_enrollment',
            field=models.BooleanField(default=False, help_text='Allows learners enrolling through learning platforms to select the audit track.', verbose_name='Enable audit enrollment for learning platform learners'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_browse_and_request',
            field=models.BooleanField(default=False, verbose_name='Display browse and request management settings'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_data_sharing_consent',
            field=models.BooleanField(default=False, help_text='Enables data sharing consent prompt for learners each time they enroll in a course. If left unchecked, the prompt will not appear and relevant data will not be shared.', verbose_name='Activate data sharing consent prompt'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_integrated_customer_learner_portal_search',
            field=models.BooleanField(default=True, help_text="Automatically enabled. If unchecked, the learners won't be able to search for a course on the learner portal.", verbose_name='Allow course discovery within the learner portal'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_learner_portal',
            field=models.BooleanField(default=True, help_text="Automatically enabled. If unchecked, learners won't have access to the learner portal."),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_learner_portal_offers',
            field=models.BooleanField(default=False, help_text='Specifies whether enterprise offers will be made known to learners in the learner portal This only applies to customers with “offers”, the old version of learner credit.', verbose_name='Enable learner credit in the learner portal'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_portal_code_management_screen',
            field=models.BooleanField(default=False, verbose_name='Display code management screen'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_portal_learner_credit_management_screen',
            field=models.BooleanField(default=False, verbose_name='Display learner credit management screen'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_portal_lms_configurations_screen',
            field=models.BooleanField(default=False, help_text='Enables the learning platform configuration screen on the administrator portal.', verbose_name='Display learning platform configuration screen'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_portal_reporting_config_screen',
            field=models.BooleanField(default=False, help_text='Enables the scheduled reporting configurations screen on the administrator portal.', verbose_name='Display enterprise reporting page'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_portal_saml_configuration_screen',
            field=models.BooleanField(default=False, help_text='Enables the Single Sign On (SSO) configuration screen on the administrator portal. ', verbose_name='Display SSO configuration screen'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_portal_subscription_management_screen',
            field=models.BooleanField(default=False, verbose_name='Display subscription management screen'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_slug_login',
            field=models.BooleanField(default=False, help_text='Allows a learner to input customer slug to identify their org in the SSO process. Should be enabled for customers that leverage SSO.', verbose_name='Allow slug login for SSO'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_universal_link',
            field=models.BooleanField(default=False, verbose_name='Display universal link settings'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enforce_data_sharing_consent',
            field=models.CharField(choices=[('at_enrollment', 'At Enrollment'), ('externally_managed', 'Managed externally')], default='at_enrollment', help_text='Setting to either require learners to accept data sharing consent at course enrollment, or through an external process.', max_length=25, verbose_name='Data sharing consent enforcement:'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='hide_course_original_price',
            field=models.BooleanField(default=False, help_text='Hides course price on learning platform course confirmation screen.', verbose_name='Hide course price on learning platform'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='hide_labor_market_data',
            field=models.BooleanField(default=False, help_text='Hides labor market data from learners (populated by features using Lightcast integration). ', verbose_name='Hide labor market data on skill features'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='replace_sensitive_sso_username',
            field=models.BooleanField(default=False, help_text='Specifies whether to replace the display of potentially sensitive SSO usernames with a more generic name, e.g. EnterpriseLearner.', verbose_name='Replace sensitive SSO username'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='reply_to',
            field=models.EmailField(blank=True, help_text='Email address that will receive learner replies to automated edX emails.', max_length=254, null=True, verbose_name='Customer “reply to” email:'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='sender_alias',
            field=models.CharField(blank=True, help_text='Specifies the sender alias for automated emails from the edX system.', max_length=255, null=True, verbose_name='Automated email sender alias'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='site',
            field=models.ForeignKey(blank=True, db_constraint=False, default=enterprise.models.get_default_site, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='sites.site'),
        ),
    ]



# Generated by Django 3.2.19 on 2023-06-08 20:41

from django.db import migrations, models
import enterprise.utils


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0173_auto_20230531_1459'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomerinvitekey',
            name='expiration_date',
            field=models.DateTimeField(default=enterprise.utils.get_default_invite_key_expiration_date, help_text='The key will no longer be valid after this date.'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomerinvitekey',
            name='expiration_date',
            field=models.DateTimeField(default=enterprise.utils.get_default_invite_key_expiration_date, help_text='The key will no longer be valid after this date.'),
        ),
    ]
